package solution;

import java.util.*;

import problem.*;

public class JoeyNode implements Comparable<JoeyNode> {
	
	private ArmConfig value;
	private Set<JoeyNode> friendList;
	private Set<JoeyNode> enemies;
	private JoeyNode parent;
	private double heuristic;
	private double travelled;
    private Map<String, Integer> edgeSizes = new HashMap<>();

	private LocalPlanner mover;
	
	JoeyNode(ArmConfig value, ArmConfig goal, ProblemSpec ps) {
		this.value = new ArmConfig(value);
		heuristic = value.getBase().distance(goal.getBase());
		mover = new LocalPlanner();
		mover.setObjects(ps.getObstacles());
		enemies = new HashSet<>();
		friendList = new HashSet<>();
	}
	
	public double seeHeuristic() {
		double ret = heuristic;
		return ret;
	}
	
	public JoeyNode getParents() {
		return parent;
	}
	
	public void setParent(JoeyNode peer) {
		this.parent = peer;
	}
	
	public boolean getTo(JoeyNode friend) {
		int edgeSize = 0;
		edgeSize = mover.edgeExists(this.value, friend.getNode());
		if (edgeSize > 0) {
			friendList.add(friend);
			friend.makeFriend(this);
			populateMap(friend, edgeSize);
			friend.populateMap(this, edgeSize);
			return true;
		} else {
			enemies.add(friend);
			friend.makeEnemies(this);
			return false;
		}
	}

	public ArmConfig getNode() {
		return this.value;
	}
	
	public void makeFriend(JoeyNode friend) {
		friendList.add(friend);
	}
	
	public void makeEnemies(JoeyNode friend) {
		enemies.add(friend);
	}
	
	public int popularity() {
		return friendList.size();
	}

	public void lookForFriends(List<JoeyNode> nodeList) {
		for (JoeyNode peer : nodeList) {
			if (!this.hates(peer) && 
					!this.friendList.contains(peer) &&
					this.value.getBase().distance(peer.getNode().getBase()) > 0.05) {
				this.getTo(peer);
			}
		}
		
	}
	
	public boolean hates(JoeyNode peer) {
		return enemies.contains(peer);
	}
	
	public int hashCode() {
		return this.value.toString().hashCode();
	}
	
	public boolean equals(Object obj) {
		if (!(obj instanceof JoeyNode)) {
			return super.equals(obj);
		}
		JoeyNode cfg = (JoeyNode) obj;
		return this.value.equals(cfg.getNode());
	}
	
	public void arrive(JoeyNode friend, int distance) {
		this.parent = friend;
		this.travelled = (double) distance + friend.getTravelled();
	}
	
	public double getTravelled() {
		return this.travelled;
	}
	
	public Set<JoeyNode> leave() {
		return friendList;
	}
	
	@Override
	public int compareTo(JoeyNode node) {
		double comp =  this.travelled + this.heuristic - 
				(node.getTravelled() + node.seeHeuristic());
		
		if (comp < 0) {
			return -1;
		} else if (comp > 0) {
			return 1;
		}
		return 0;
	}

	public int edgeSize(JoeyNode friend) {
		String one = new String(this.getNode().toString());
		String two = new String(friend.getNode().toString());
		String key1 = one + two;
		return edgeSizes.get(key1);
	}
	
	public void populateMap(JoeyNode friend, int size) {
		String one = new String(this.getNode().toString());
		String two = new String(friend.getNode().toString());
		String key1 = one + two;
		edgeSizes.put(key1, size);
	}
}
